searchData={"content_type":"text/markdown","items":[{"doc":"Callback behaviours and functions for WebSocket client.","ref":"Fresh.html","title":"Fresh","type":"behaviour"},{"doc":"This macro simplifies the implementation of WebSocket client. It automatically configures `child_spec/1`, `start/1` and `start_link/1` for the module, and provides empty handlers for all required callbacks, which can be overridden.\n\nStarting the WebSocket client using `start_link/1` with the desired options:\n\n    iex> Example.WebSocket.start_link(uri: \"wss://example.com/socket\", state: %{}, opts: [\n    iex>   name: {:local, :ws_conn}\n    iex> ])\n    {:ok, #PID }\n\n\nStarting the WebSocket client using Supervisor (recommended):\n\n    children = [\n      {Example.WebSocket,\n       uri: \"wss://example.com/socket\",\n       state: %{},\n       opts: [\n         name: {:local, :ws_conn}\n       ]}\n      # Add other child specifications...\n    ]\n\n    Supervisor.start_link(children, strategy: :one_for_one)","ref":"Fresh.html#__using__/1","title":"Fresh.__using__/1","type":"macro"},{"doc":"Sends a WebSocket close frame to the server.\n\n- `pid`: The WebSocket connection process.\n- `code`: WebSocket close code.\n\n  Example: `1000`\n\n- `reason`: WebSocket close reason.\n\n  Example: `\"dunno\"`","ref":"Fresh.html#close/3","title":"Fresh.close/3","type":"function"},{"doc":"iex> Fresh.close(:ws_conn, 1000, \"dunno\")\n    :ok","ref":"Fresh.html#close/3-example","title":"Example - Fresh.close/3","type":"function"},{"doc":"Callback is invoked when a WebSocket connection is successfully established.\n\n- `status`: The status received during the connection upgrade.\n- `headers`: The headers received during the connection upgrade.\n\n  Example: `[{\"upgrade\", \"websocket\"}, {\"connection\", \"upgrade\"}]`\n\n- `state`: The current state of the module.","ref":"Fresh.html#c:handle_connect/3","title":"Fresh.handle_connect/3","type":"callback"},{"doc":"def handle_connect(_status, _headers, state) do\n      payload = \"{ \\\"op\\\": 1, \\\"data\\\": {} }\"\n      {:reply, [{:text, payload}], state}\n    end","ref":"Fresh.html#c:handle_connect/3-example","title":"Example - Fresh.handle_connect/3","type":"callback"},{"doc":"Callback is invoked when a control frame is received from the server.\n\n- `frame`: The received WebSocket frame.\n- `state`: The current state of the module.","ref":"Fresh.html#c:handle_control/2","title":"Fresh.handle_control/2","type":"callback"},{"doc":"def handle_control({:ping, message}, state) do\n      IO.puts(\"Received ping with content: #{message}!\")\n      {:ok, state}\n    end\n\n    def handle_control({:pong, message}, state) do\n      IO.puts(\"Received pong with content: #{message}!\")\n      {:ok, state}\n    end","ref":"Fresh.html#c:handle_control/2-example","title":"Example - Fresh.handle_control/2","type":"callback"},{"doc":"Callback is invoked when the WebSocket connection is being disconnected.\n\n- `code`: The disconnection code (if available).\n\n  Example: `1002`\n\n- `reason`: The reason for the disconnection (if available).\n\n  Example: `\"timeout\"`\n\n- `state`: The current state of the module.","ref":"Fresh.html#c:handle_disconnect/3","title":"Fresh.handle_disconnect/3","type":"callback"},{"doc":"def handle_disconnect(1002, _reason, _state), do: :reconnect\n    def handle_disconnect(_code, _reason, _state), do: :close","ref":"Fresh.html#c:handle_disconnect/3-example","title":"Example - Fresh.handle_disconnect/3","type":"callback"},{"doc":"Callback is invoked when an error occurs, allows you to define custom error handling logic to handle various scenarios gracefully.\n\n- `error`: The encountered error.\n- `state`: The current state of the module.","ref":"Fresh.html#c:handle_error/2","title":"Fresh.handle_error/2","type":"callback"},{"doc":"def handle_error({error, _reason}, state)\n        when error in [:encoding_failed, :casting_failed],\n        do: {:ignore, state}\n\n    def handle_error(_error, _state), do: :reconnect","ref":"Fresh.html#c:handle_error/2-example","title":"Example - Fresh.handle_error/2","type":"callback"},{"doc":"Callback is invoked when a data frame is received from the server.\n\n- `frame`: The received WebSocket frame.\n- `state`: The current state of the module.","ref":"Fresh.html#c:handle_in/2","title":"Fresh.handle_in/2","type":"callback"},{"doc":"def handle_in({:text, message}, state) do\n      %{\"data\" => updated_data} = Jason.decode!(message)\n      {:ok, updated_data}\n    end\n\n    def handle_in({:binary, _message}, state) do\n      {:reply, [{:text, \"i don't accept binary!\"}], state}\n    end","ref":"Fresh.html#c:handle_in/2-example","title":"Example - Fresh.handle_in/2","type":"callback"},{"doc":"Callback is invoked when an incomprehensible message is received.\n\n- `data`: The received message.\n- `state`: The current state of the module.","ref":"Fresh.html#c:handle_info/2","title":"Fresh.handle_info/2","type":"callback"},{"doc":"def handle_info({:reply, message}, state) do\n      {:reply, [{:text, message}], state}\n    end\n\nCan be used like:\n\n    send(:ws_conn, {:reply, \"hello!\"})","ref":"Fresh.html#c:handle_info/2-example","title":"Example - Fresh.handle_info/2","type":"callback"},{"doc":"Callback is invoked when the process is about to terminate.\n\n- `reason`: The reason for the termination.\n\n  Example: `:normal`, `:shutdown`, `{:error, :unknown_data}`\n\n- `state`: The current state of the module.\n\nReturn value is always ignored.","ref":"Fresh.html#c:handle_terminate/2","title":"Fresh.handle_terminate/2","type":"callback"},{"doc":"def handle_terminate(reason, _state) do\n      IO.puts(\"Process is terminating with reason: #{inspect(reason)}\")\n    end","ref":"Fresh.html#c:handle_terminate/2-example","title":"Example - Fresh.handle_terminate/2","type":"callback"},{"doc":"Sends a WebSocket frame to the server.\n\n- `pid`: The WebSocket connection process.\n- `frame`: The WebSocket frame to send.\n\n  Example: `{:text, \"hi!\"}`","ref":"Fresh.html#send/2","title":"Fresh.send/2","type":"function"},{"doc":"iex> Fresh.send(:ws_conn, {:text, \"hi!\"})\n    :ok","ref":"Fresh.html#send/2-example","title":"Example - Fresh.send/2","type":"function"},{"doc":"Starts a WebSocket connection without linking process. Refer to `start_link/4` for more information.","ref":"Fresh.html#start/4","title":"Fresh.start/4","type":"function"},{"doc":"Starts a WebSocket connection.\n\n- `uri`: The URI to connect to.\n\n  Example: `\"wss://example.com/socket\"`\n\n- `module`: The module that implementes the WebSocket client behaviour.\n\n  Example: `Example.WebSocket`\n\n- `state`: The initial state to be passed to the module when it starts.\n\n  Example: `%{}`\n\n- `opts`: A list of options to configure the WebSocket connection. Refer to `t:option/0` for available options.\n\n  Example: `[name: {:local, :ws_conn}, headers: [{\"Authorization\", \"Bearer token\"}]]`","ref":"Fresh.html#start_link/4","title":"Fresh.start_link/4","type":"function"},{"doc":"iex> Fresh.start_link(\"wss://example.com/socket\", Example.WebSocket, %{}, name: {:local, :ws_conn})\n    {:ok, #PID }","ref":"Fresh.html#start_link/4-example","title":"Example - Fresh.start_link/4","type":"function"},{"doc":"Represents the result of a connection handle (disconnect, error) callback.\n\n- `{:ignore, state}`: Indicates an intent to ignore and continue.\n\n  Example: `{:ignore, state}`\n\n- `{:reconnect, state}`: Indicates an intent to reconnect, with given state.\n\n  Example: `{:reconnect, 0}`\n\n- `{:close, reason}`: Indicates an intent to close connection, with custom reason. Can be used to force Supervisor to restart process.\n\n  Example: `{:close, {:error, :unknown}}`\n\n- `:reconnect`: Indicates an intent to reconnect, with initial state.\n- `:close`: Indicates an intent to close connection normally.","ref":"Fresh.html#t:connection_handle_response/0","title":"Fresh.connection_handle_response/0","type":"type"},{"doc":"Represents control frames in a WebSocket connection.","ref":"Fresh.html#t:control_frame/0","title":"Fresh.control_frame/0","type":"type"},{"doc":"Represents data frames in a WebSocket connection.","ref":"Fresh.html#t:data_frame/0","title":"Fresh.data_frame/0","type":"type"},{"doc":"Represents various error scenarios that can occur during WebSocket communication.","ref":"Fresh.html#t:error/0","title":"Fresh.error/0","type":"type"},{"doc":"Represents the result of a generic callback.\n\n- `{:ok, state}`: Indicates a successful operation, and it updates the state of the module.\n\n  Example: `{:ok, state + 1}`, `{:ok, state}`\n\n- `{:reply, frames, state}`: Indicates a successful operation with a reply to the server, and it updates the state of the module.\n\n  Example: `{:reply, [{:text, \"Bousou\"}, {:text, \"暴走\"}], state + 1}`","ref":"Fresh.html#t:generic_handle_response/0","title":"Fresh.generic_handle_response/0","type":"type"},{"doc":"Represents optional configurations for WebSocket configuration.\n\n- `:name`: Registers a name for the WebSocket connection, allowing you to refer to it later using a name.\n\n  Example: `{:local, Example.WebSocket}`\n\n- `:headers`: A list of headers to include in the WebSocket connection request. These headers will be sent during the connection upgrade.\n\n  Example: `{:headers, [{\"Authorization\", \"Bearer token\"}]}`\n\n- `:transport_opts`: Additional options to pass to the transport layer used for the WebSocket connection. Consult the [Mint.HTTP documentation](https://hexdocs.pm/mint/Mint.HTTP.html#connect/4-options) for more informations.\n\n  Example: `{:transport_opts, [cacertfile: \"/my/file\"]}`\n\n- `:mint_upgrade_opts`: Extra options to provide to [Mint.WebSocket](https://github.com/elixir-mint/mint_web_socket) during the WebSocket upgrade process. Consult the [Mint.WebSocket documentation](https://hexdocs.pm/mint_web_socket/Mint.WebSocket.html#upgrade/5-options) for additional information.\n\n  Example: `{:mint_upgrade_opts, [extensions: [Mint.WebSocket.PerMessageDeflate]]}`\n\n- `:ping_interval`: This option is used to keep connection alive by sending empty ping frames based on given time as milliseconds. By default it is set to `30000`. To disable it, set it to `0`.\n\n  Example: `{:ping_interval, 60_000}`\n\n- `:error_logging`: Allowing you to turn on/off logging `t:error/0` errors. Enabled by default.\n\n  Example: `{:error_logging, false}`","ref":"Fresh.html#t:option/0","title":"Fresh.option/0","type":"type"},{"doc":"Represents the state of the given module, which can be anything.","ref":"Fresh.html#t:state/0","title":"Fresh.state/0","type":"type"},{"doc":"# Changelog\n\nAll notable changes to this project will be documented in this file.\n\nThe format is based on [Keep a Changelog](https://keepachangelog.com/en/1.1.0/),\nand this project adheres to [Semantic Versioning](https://semver.org/spec/v2.0.0.html).\n\n## [Unreleased]\n\n## [0.2.1] - 2023-09-17\n\n#","ref":"changelog.html","title":"Changelog","type":"extras"},{"doc":"- `close/3` Function for sending close frame easier.\n\n## [0.2.0] - 2023-09-14\n\n#","ref":"changelog.html#added","title":"Added - Changelog","type":"extras"},{"doc":"- `handle_terminate/2` Callback for process termination.\n\n#","ref":"changelog.html#added","title":"Added - Changelog","type":"extras"},{"doc":"- Minor documentation issues.\n\n## [0.1.1] - 2023-09-12\n\n#","ref":"changelog.html#fixed","title":"Fixed - Changelog","type":"extras"},{"doc":"- Add queue for incoming messages while websocket is `nil`.\n\n## [0.1.0] - 2023-09-11\n\n#","ref":"changelog.html#fixed","title":"Fixed - Changelog","type":"extras"},{"doc":"- Missing `start/1` for `__using__/1` macro.\n\n## [0.1.0-rc] - 2023-09-09\n\n- Initial release of Fresh.","ref":"changelog.html#added","title":"Added - Changelog","type":"extras"},{"doc":"# Fresh\n\nFresh is an attempt to create a simple, reliable and flexible WebSocket client built atop the [Mint](https://github.com/elixir-mint) ecosystem 🌱","ref":"readme.html","title":"Introduction","type":"extras"},{"doc":"Discover the reasons behind choosing Fresh over existing libraries, summed up in three key aspects:\n\n- Simplicity\n- Resilience\n- Control\n\n#","ref":"readme.html#why-fresh","title":"Why Fresh? - Introduction","type":"extras"},{"doc":"Fresh is designed with simplicity in mind, offering a user-friendly API that includes clear examples and comprehensive documentation.\n\n#","ref":"readme.html#simplicity","title":"Simplicity - Introduction","type":"extras"},{"doc":"Fresh excels in ensuring a robust and enduring connection to the server. By default, Fresh promptly re-establishing connection when the server terminates the connection or encounters any connectivity issues. When used alongside Supervisor, Fresh delivers exceptional reliability.\n\n#","ref":"readme.html#resilience","title":"Resilience - Introduction","type":"extras"},{"doc":"With Fresh, you gain extensive control over the flow of WebSocket connections, including control over resilience. You can manage scenarios requiring reconnection, identify those that should be ignored, specify when to gracefully terminate a connection, and even instruct Supervisor to restart your process as needed. Additionally, Fresh allows you to leverage custom `Mint.WebSocket` options, transport layer options, custom headers, and more, providing you with the flexibility you require.","ref":"readme.html#control","title":"Control - Introduction","type":"extras"},{"doc":"Package can be installed by adding `fresh` to your list of dependencies in `mix.exs`:\n\n```elixir\ndefp deps do\n  [\n    {:fresh, \"~> 0.2.1\"}\n  ]\nend\n```\n\n#","ref":"readme.html#installation","title":"Installation - Introduction","type":"extras"},{"doc":"This library is well-tested with the following versions of Elixir and Erlang/OTP:\n\n- Elixir 1.14 or newer\n- Erlang/OTP 25 or newer\n\nWhile it may also work with older versions, we strongly recommend using the specified minimum versions for the best experience.","ref":"readme.html#compatibility","title":"Compatibility - Introduction","type":"extras"},{"doc":"Below is an example of a WebSocket client that handles incoming frames to implement a simple counter:\n\n_Need to mention this example module contains approximately 15 lines of code._\n\n```elixir\ndefmodule EchoWebSocket do\n  use Fresh\n\n  def handle_connect(_status, _headers, _state) do\n    IO.puts(\"Start counting from 0\")\n    {:reply, [{:text, \"1\"}], 0}\n  end\n\n  def handle_in({:text, number}, _state) do\n    number = String.to_integer(number)\n\n    IO.puts(\"Number: #{number}\")\n    {:reply, [{:text, \"#{number + 1}\"}], number}\n  end\nend\n```\n\nMore examples can be found inside [examples/](https://github.com/bunopnu/fresh/tree/main/examples) folder.","ref":"readme.html#example","title":"Example - Introduction","type":"extras"},{"doc":"Check out [HexDocs website](https://hexdocs.pm/fresh) for documentation and API reference.","ref":"readme.html#documentation","title":"Documentation - Introduction","type":"extras"},{"doc":"Fresh is licensed under the MIT license.","ref":"readme.html#license","title":"License - Introduction","type":"extras"},{"doc":"Copyright (c) 2023 bunopnu   (https://bun.rip)\n\nPermission is hereby granted, free of charge, to any person obtaining a copy\nof this software and associated documentation files (the \"Software\"), to deal\nin the Software without restriction, including without limitation the rights\nto use, copy, modify, merge, publish, distribute, sublicense, and/or sell\ncopies of the Software, and to permit persons to whom the Software is\nfurnished to do so, subject to the following conditions:\n\nThe above copyright notice and this permission notice shall be included in all\ncopies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\nIMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\nFITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\nAUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\nLIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,\nOUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE\nSOFTWARE.","ref":"license.html","title":"License","type":"extras"}]}